/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetCompAnalogSensor.c

Abstract:

	This file contains methods for a Phidget Component AnalogSensor. 
	A component AnalogSensor can belong to several phidgets e.g. the phidget
	InterfaceKit. Only if one of these phidgets is attached the component is
	useable. Use phidget_GetDeviceClass(pointerToPhidget) to call the
	specific phidget methods in phidget21 API.

--*/

#include "PhidgetCompAnalogSensor.h"

//////////////////////////////////////////////////////////////////////////
// methods

//
// Gets the number of analog sensor inputs supported by the owning phidget
// of this component. Use id to identify the owning phidget.
//
int phidgetCompAnalogSensor_GetCount(PhidgetId id)
{
	int iCount = 0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, iCount);
	
	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
		case PHIDCLASS_INTERFACEKIT:
			CPhidgetInterfaceKit_getSensorCount ( ((PhidgetIfKit*) pPhidget)->handle, &iCount);
			break;
			
		default:
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAnalogSensor_GetCount.", phidget_GetDeviceClass(pPhidget)));
	}
	
	return iCount;
}

//
// Gets the current sensor value of a component analog sensor with the
// corresponding index. Use id to identify the owning phidget.
//
int phidgetCompAnalogSensor_GetValue(PhidgetId id, int iIndex)
{
	int iValue = 0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, iValue);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
		case PHIDCLASS_INTERFACEKIT:
			CPhidgetInterfaceKit_getSensorValue ( ((PhidgetIfKit*) pPhidget)->handle, iIndex, &iValue);
			break;
			
		default:
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAnalogSensor_GetValue.", phidget_GetDeviceClass(pPhidget)));
	}
	
	return iValue;
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has changed a components analog sensor value. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreSensorChange is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompAnalogSensor_SetSemaphoreIndex_ValueChange(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_INTERFACEKIT:
		((PhidgetIfKit*) pPhidget)->iSemaphoreSensorChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <IfKit-ValueChange> is: %d, Phidget id = %i",
			((PhidgetIfKit*) pPhidget)->iSemaphoreSensorChange, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAnalogSensor_SetSemaphoreIndex_ValueChange.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}
