/*++
	
Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetCompAdvancedServoMotor.c

Abstract:

	This file contains methods for a Phidget Component AdvancedServoMotor which is
	a special ServoMotor. 
	A component AdvancedServoMotor can belong to several phidgets e.g. the phidget
	AdvancedServo. Only if one of these phidgets is attached the component is
	useable. Use phidget_GetDeviceClass(pointerToPhidget) to call the specific 
	phidget methods	in phidget21 API.

--*/

#include "PhidgetCompAdvancedServoMotor.h"

//////////////////////////////////////////////////////////////////////////
// methods

//
// Gets the current velocity limit of a component advanced servo motor
// with the corresponding index. Use id to identify the owning phidget.
//
double phidgetCompAdvancedServoMotor_GetVelocityLimit(PhidgetId id, int iIndex)
{
	double dLimit = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dLimit);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getVelocityLimit(((PhidgetAdvServo*) pPhidget)->handle, iIndex, &dLimit);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_GetVelocityLimit.", phidget_GetDeviceClass(pPhidget)));
	}

	return dLimit;
}

//
// Sets the velocity limit of a component advanced servo motor with the
// corresponding index. Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompAdvancedServoMotor_SetVelocityLimit(PhidgetId id, int iIndex, double dLimit)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		return CPhidgetAdvancedServo_setVelocityLimit(((PhidgetAdvServo*) pPhidget)->handle, iIndex, dLimit ) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_GetVelocityLimit.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Gets the current velocity of a component advanced servo motor with
// the corresponding index. Use id to identify the owning phidget.
//
double phidgetCompAdvancedServoMotor_GetVelocity(PhidgetId id, int iIndex)
{
	double dVelocity = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dVelocity);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getVelocity(((PhidgetAdvServo*) pPhidget)->handle, iIndex, &dVelocity);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_GetVelocityLimit.", phidget_GetDeviceClass(pPhidget)));
	}

	return dVelocity;
}

//
// Gets the current maximum velocity supported by a component advanced
// servo motor with the corresponding index. Use id to identify the
// owning phidget.
//
double phidgetCompAdvancedServoMotor_GetVelocityMax(PhidgetId id, int iIndex)
{
	double dVelocity = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dVelocity);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getVelocityMax(((PhidgetAdvServo*) pPhidget)->handle, iIndex, &dVelocity);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_GetVelocityMax.", phidget_GetDeviceClass(pPhidget)));
	}

	return dVelocity;
}

//
// Gets the current minimum velocity supported by a component advanced
// servo motor with the corresponding index. Use id to identify the
// owning phidget.
//
double phidgetCompAdvancedServoMotor_GetVelocityMin(PhidgetId id, int iIndex)
{
	double dVelocity = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dVelocity);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getVelocityMin(((PhidgetAdvServo*) pPhidget)->handle, iIndex, &dVelocity);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_GetVelocityMin.", phidget_GetDeviceClass(pPhidget)));
	}

	return dVelocity;
}

//
// Sets the maximal position of a component advanced servo motor with
// the corresponding index. Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompAdvancedServoMotor_SetPositionMax(PhidgetId id, int iIndex, double dPosition)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		return CPhidgetAdvancedServo_setPositionMax(((PhidgetAdvServo*) pPhidget)->handle, iIndex, dPosition ) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_SetPositionMax.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Sets the minimal position of a component advanced servo motor with
// the corresponding index. Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompAdvancedServoMotor_SetPositionMin(PhidgetId id, int iIndex, double dPosition)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		return CPhidgetAdvancedServo_setPositionMin(((PhidgetAdvServo*) pPhidget)->handle, iIndex, dPosition ) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_SetPositionMin.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Gets the current acceleration of a component advanced servo motor
// with the corresponding index. Use id to identify the owning phidget.
//
double phidgetCompAdvancedServoMotor_GetAcceleration(PhidgetId id, int iIndex)
{
	double dAcceleration = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dAcceleration);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getAcceleration(((PhidgetAdvServo*) pPhidget)->handle, iIndex, &dAcceleration);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_GetAcceleration.", phidget_GetDeviceClass(pPhidget)));
	}

	return dAcceleration;
}

//
// Sets the acceleration of a component advanced servo motor with the
// corresponding index. Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompAdvancedServoMotor_SetAcceleration(PhidgetId id, int iIndex, double dAcceleration)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		return CPhidgetAdvancedServo_setAcceleration(((PhidgetAdvServo*) pPhidget)->handle, iIndex, dAcceleration ) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_SetAcceleration.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Gets the minimum accleration supported by a component advanced servo
// motor with the corresponding index. Use id to identify the owning
// phidget.
//
double phidgetCompAdvancedServoMotor_GetAccelerationMin(PhidgetId id, int iIndex)
{
	double dAcceleration = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dAcceleration);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getAccelerationMin(((PhidgetAdvServo*) pPhidget)->handle, iIndex, &dAcceleration);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_GetAccelerationMin.", phidget_GetDeviceClass(pPhidget)));
	}

	return dAcceleration;
}

//
// Gets the maximum accleration supported by a component advanced servo
// motor with the corresponding index. Use id to identify the owning
// phidget.
//
double phidgetCompAdvancedServoMotor_GetAccelerationMax(PhidgetId id, int iIndex)
{
	double dAcceleration = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dAcceleration);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getAccelerationMax(((PhidgetAdvServo*) pPhidget)->handle, iIndex, &dAcceleration);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_GetAccelerationMax.", phidget_GetDeviceClass(pPhidget)));
	}

	return dAcceleration;
}

//
// Gets the current current draw of a component advanced servo motor with
// the corresponding index. Use id to identify the owning phidget.
//
double phidgetCompAdvancedServoMotor_GetCurrent(PhidgetId id, int iIndex)
{
	double dCurrent = 0.0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dCurrent);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getCurrent(((PhidgetAdvServo*) pPhidget)->handle, iIndex, &dCurrent);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_GetCurrent.", phidget_GetDeviceClass(pPhidget)));
	}

	return dCurrent;
}

//
// Gets the current speed ramping on state of a component servo motor with
// the corresponding index. This is true when velocity an acceleration are
// used otherwise false.
// Use id to identify the owning phidget.
//
BOOL phidgetCompAdvancedServoMotor_IsSpeedRampingOn(PhidgetId id, int iIndex)
{
	BOOL bState = FALSE;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, bState);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getSpeedRampingOn( ((PhidgetAdvServo*) pPhidget)->handle, iIndex, &bState );
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_IsSpeedRampingOn.", phidget_GetDeviceClass(pPhidget)));
	}

	return bState > 0;
}

//
// Sets the speed ramping on state of a component advanced servo motor
// with the corresponding index. Set to true if acceleration and
// velocity should be used.
// Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompAdvancedServoMotor_SetSpeedRampingOn(PhidgetId id, int iIndex, BOOL bState)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		return CPhidgetAdvancedServo_setSpeedRampingOn(((PhidgetAdvServo*) pPhidget)->handle, iIndex, bState ) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_SetSpeedRampingOn.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Gets the current stopped state of a component servo motor with the
// corresponding index. This is true when the motor is not moving and
// there are no outstanding commands.
// Use id to identify the owning phidget.
//
BOOL phidgetCompAdvancedServoMotor_IsStopped (PhidgetId id, int iIndex)
{
	BOOL bStopped = FALSE;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, bStopped);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		CPhidgetAdvancedServo_getStopped( ((PhidgetAdvServo*) pPhidget)->handle, iIndex, &bStopped );
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_IsStopped.", phidget_GetDeviceClass(pPhidget)));
	}

	return bStopped > 0;
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has changed a components servo motor velocity. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreVelocityChange is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompAdvancedServoMotor_SetSemaphoreIndex_VelocityChange(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		((PhidgetAdvServo*) pPhidget)->iSemaphoreVelocityChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <AdvServo-VelocityChange> is: %d, Phidget id = %i",
			((PhidgetAdvServo*) pPhidget)->iSemaphoreVelocityChange, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_SetSemaphoreIndex_VelocityChange.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has changed a components servo motor current draw. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreCurrentChange is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompAdvancedServoMotor_SetSemaphoreIndex_CurrentChange(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ADVANCEDSERVO:
		((PhidgetAdvServo*) pPhidget)->iSemaphoreCurrentChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <AdvServo-CurrentChange> is: %d, Phidget id = %i",
			((PhidgetAdvServo*) pPhidget)->iSemaphoreCurrentChange, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompAdvancedServoMotor_SetSemaphoreIndex_CurrentChange.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}
