/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetCompRFID.c

Abstract:

	This file contains methods for a Phidget Component RFID. 
	A component RFID can belong to several phidgets e.g. the phidget RFID. Only if
	one of these phidgets is attached the component is useable.	Use
	phidget_GetDeviceClass(pointerToPhidget) to call the specific phidget methods
	in phidget21 API.

--*/

#include "PhidgetCompRFID.h"

//////////////////////////////////////////////////////////////////////////
// methods

//
// Gets the current antenna state of a component rfid with the
// corresponding index. This is true when the antenna is enabled. Note that
// the antenna must be enabled before tags will be read.
// Use id to identify the owning phidget.
//
BOOL phidgetCompRFID_IsAntennaOn(PhidgetId id)
{
	BOOL bState = FALSE;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, bState);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_RFID:
		CPhidgetRFID_getAntennaOn ( ((PhidgetRFID*) pPhidget)->handle, &bState);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompRFID_IsAntennaOn.", phidget_GetDeviceClass(pPhidget)));
	}

	return bState;
}

//
// Sets the antenna state of a component rfid with the corresponding
// index. Set to true if the antenna should be enabled (necassary to find
// tags.
// Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompRFID_SetAntennaState(PhidgetId id, BOOL bState)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_RFID:
		return CPhidgetRFID_setAntennaOn ( ((PhidgetRFID*) pPhidget)->handle, bState) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompRFID_SetAntennaState.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Gets the last tag of a component rfid with the corresponding index.
// This tag may or may not still be on the reader. Return TRUE if the
// getting succeded, otherwise false.
// Use id to identify the owning phidget.
//
BOOL phidgetCompRFID_GetLastTag(PhidgetId id, char* pcTagArray)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);
	if (pcTagArray == NULL) return FALSE;

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_RFID:
		return CPhidgetRFID_getLastTag( ((PhidgetRFID*) pPhidget)->handle, (unsigned char*) pcTagArray ) == 0;
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompRFID_GetLastTag.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

//
// Gets the current tag present status of a component rfid with the
// corresponding index. This is true when a tag is being read by the reader.
// Use id to identify the owning phidget.
//
BOOL phidgetCompRFID_HasTag(PhidgetId id)
{
	BOOL bState = FALSE;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, bState);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_RFID:
		CPhidgetRFID_getTagStatus ( ((PhidgetRFID*) pPhidget)->handle, &bState);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompRFID_HasTag.", phidget_GetDeviceClass(pPhidget)));
	}

	return bState;
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has first detected a tag by the reader. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreTagFound is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompRFID_SetSemaphoreIndex_TagFound(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_RFID:
		((PhidgetRFID*) pPhidget)->iSemaphoreTagFound = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <RFID-TagFound> is: %d, Phidget id = %i",
			((PhidgetRFID*) pPhidget)->iSemaphoreTagFound, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompRFID_SetSemaphoreIndex_TagFound.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id is no longer detecting a tag by the reader. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreTagLost is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompRFID_SetSemaphoreIndex_TagLost(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_RFID:
		((PhidgetRFID*) pPhidget)->iSemaphoreTagLost = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <RFID-TagLost> is: %d, Phidget id = %i",
			((PhidgetRFID*) pPhidget)->iSemaphoreTagLost, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompRFID_SetSemaphoreIndex_TagLost.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}
