/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

--*/

#include "PhidgetManager.h"

//////////////////////////////////////////////////////////////////////////
// some mutex macros

#define LOCK_MUTEX() \
	{ if(!g_bPhidgetManagerLockInitialised) {		\
		xmutex_create(&g_phidgetManagerMutex);		\
		g_bPhidgetManagerLockInitialised = TRUE; }	\
	xmutex_lock(&g_phidgetManagerMutex); }
#define UNLOCK_MUTEX() \
	{ xmutex_unlock(&g_phidgetManagerMutex); }

//////////////////////////////////////////////////////////////////////////
// Types

// entry in an attached devices list
typedef struct _DeviceListEntry 
{
	CPhidgetHandle				handle;			// handle to the phidget
	struct _DeviceListEntry *	pNext;			// pointer to next item in list, or NULL if it is the last
} DeviceListEntry;

// data the phidget manager has to know
typedef struct _PhidgetManagerData
{
	CPhidgetManagerHandle	phidgetManager;		// handle to phidget21's Phidget Manager
	DeviceListEntry*		attachedDevices;	// pointer to first item of the list of attached devices, or NULL if it is empty
	int						iListSize;			// size of latter list
	BOOL					bListInitialised;	// shows whether reset... should be called
	int						iSemaphoreAttach;	// global semaphores fpr phidget manager
	int						iSemaphoreDetach;	// ...
	int						iSemaphoreError;	// ...
} PhidgetManagerData;


//////////////////////////////////////////////////////////////////////////
// Global variables

PhidgetManagerData	g_phidgetManagerData;				// data the phidget manager has to know
xmutex_t			g_phidgetManagerMutex;				// mutex
BOOL				g_bPhidgetManagerLockInitialised;	// ...

//////////////////////////////////////////////////////////////////////////
// forward declarations

void phidgetManager_resetSemaphores();
BOOL phidgetManager_initaliseAttachedDevicesList();
BOOL phidgetManager_freeAttachedDevicesList();
BOOL phidgetManager_addPhidgetToList(CPhidgetHandle handle);

int CALLBACK phidgetManager_attachHandler(CPhidgetHandle newPhidget, void* pUserPtr);
int CALLBACK phidgetManager_detachHandler(CPhidgetHandle newPhidget, void* pUserPtr);
int CALLBACK phidgetManager_errorHandler(CPhidgetManagerHandle manager, void* pUserPtr, int iError, const char* pcError);

//////////////////////////////////////////////////////////////////////////
// function definitions

// initialise PhidgetManager
// Note: This function should not be called from more than 1 thread concurrently.
BOOL phidgetManager_initialise()
{
	LOCK_MUTEX();

	// clear data
	memset(&g_phidgetManagerData, 0, sizeof(g_phidgetManagerData));
	phidgetManager_resetSemaphores();
	g_phidgetManagerData.bListInitialised = FALSE;

	// create handle
	if(CPhidgetManager_create(&g_phidgetManagerData.phidgetManager) != 0)
	{
		g_phidgetManagerData.phidgetManager = NULL;
		UNLOCK_MUTEX();

		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Could not create phidget manager."));
		return FALSE;
	}

	// set event handlers
	CPhidgetManager_set_OnAttach_Handler(g_phidgetManagerData.phidgetManager, phidgetManager_attachHandler, NULL);
	CPhidgetManager_set_OnDetach_Handler(g_phidgetManagerData.phidgetManager, phidgetManager_detachHandler, NULL);
	CPhidgetManager_set_OnError_Handler(g_phidgetManagerData.phidgetManager, phidgetManager_errorHandler, NULL);
	
	// open handle
	if(CPhidgetManager_open(g_phidgetManagerData.phidgetManager) != 0)
	{
		CPhidgetManager_delete(g_phidgetManagerData.phidgetManager);
		g_phidgetManagerData.phidgetManager = NULL;
		UNLOCK_MUTEX();

		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Could not open phidget manager."));
		return FALSE;
	}
	else
	{
		IF_DBG(SLog(SLOG_LEVEL_INFO, "PhidgetManager is initialised."));

		// initialise the list
		if(!phidgetManager_initaliseAttachedDevicesList())
			IF_DBG(SLog(SLOG_LEVEL_WARNING, "Could not initialise the attached devices list."));

		UNLOCK_MUTEX();
		return TRUE;
	}
}

// close PhidgetManager
BOOL phidgetManager_shutDown()
{
	LOCK_MUTEX();

	// close phidget manager handle
	if(g_phidgetManagerData.phidgetManager == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Trying to shutdown phidget manager, but it is not initialised."));
	}
	else
	{
		CPhidgetManager_close(g_phidgetManagerData.phidgetManager);
		CPhidgetManager_delete(g_phidgetManagerData.phidgetManager);
		g_phidgetManagerData.phidgetManager = NULL;
	}

	// clear list and reset semaphores
	phidgetManager_freeAttachedDevicesList();
	phidgetManager_resetSemaphores();

	// but we can leave the mutex initialised
	UNLOCK_MUTEX();
	return TRUE;
}

void phidgetManager_resetSemaphores()
{
	LOCK_MUTEX();

	g_phidgetManagerData.iSemaphoreAttach = 0;
	g_phidgetManagerData.iSemaphoreDetach = 0;
	g_phidgetManagerData.iSemaphoreError = 0;

	UNLOCK_MUTEX();
}

// reset the global list of attached devices
BOOL phidgetManager_initaliseAttachedDevicesList()
{
	CPhidgetHandle* list = NULL;
	int iCount = 0;
	int i;

	LOCK_MUTEX();

	if(g_phidgetManagerData.bListInitialised)
	{
		UNLOCK_MUTEX();
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "Trying to initialise an already initialised list of attached devices."));
		return FALSE;
	}

	// get the list from phidget21
	if( g_phidgetManagerData.phidgetManager == NULL || 
		CPhidgetManager_getAttachedDevices(g_phidgetManagerData.phidgetManager, &list, &iCount) != EPHIDGET_OK)
	{
		UNLOCK_MUTEX();
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Could not get the list of attached devices from phidget manager."));
		return FALSE;
	}

	// set as initialised
	g_phidgetManagerData.bListInitialised = TRUE;
	UNLOCK_MUTEX();
	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "Got list of attached phidgets. Will add them now... Count = %i", iCount));

	// save each item in the global list
	for(i = 0; i < iCount; i++)
		phidgetManager_addPhidgetToList(list[i]);
	
	// clean up
	CPhidgetManager_freeAttachedDevicesArray(list);

	return TRUE;
}

BOOL phidgetManager_freeAttachedDevicesList()
{
	DeviceListEntry *pIterator, *pNext;

	LOCK_MUTEX();

	if(!g_phidgetManagerData.bListInitialised)
	{
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "Trying to free an already freed list of attached devices."));
		return FALSE;
	}

	// free data
	for(pIterator = g_phidgetManagerData.attachedDevices; pIterator != NULL; pIterator = pNext)
	{
		pNext = pIterator->pNext;
		free(pIterator);
	}

	// reset attributes
	g_phidgetManagerData.attachedDevices = NULL;
	g_phidgetManagerData.iListSize = 0;
	g_phidgetManagerData.bListInitialised = FALSE;

	UNLOCK_MUTEX();
	return TRUE;
}

BOOL phidgetManager_addPhidgetToList(CPhidgetHandle handle)
{
	DeviceListEntry* pItem;

	// if the list is not initialised, initialise it now.
	// if we do so, the new entry is automatically added. else we need to add it manually.

	LOCK_MUTEX();

	if(!g_phidgetManagerData.bListInitialised)
		phidgetManager_initaliseAttachedDevicesList();
	else // add manually
	{
		IF_DBG(SLog(SLOG_LEVEL_INFO, "Adding new Phidget to attached list. handle = %i", handle));

		// allocate some memory for the new entry
		pItem = malloc(sizeof(DeviceListEntry));
		if(pItem == NULL)
		{
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot allocate %i bytes in phidgetManager_addPhidgetToList", sizeof(DeviceListEntry)));
			UNLOCK_MUTEX();
			return FALSE;
		}

		// set handle and add item at the beginning
		pItem->handle = handle;

		pItem->pNext = g_phidgetManagerData.attachedDevices;
		g_phidgetManagerData.attachedDevices = pItem;
		g_phidgetManagerData.iListSize++;
	}

	UNLOCK_MUTEX();
	return TRUE;
}

BOOL phidgetManager_removePhidgetFromList(CPhidgetHandle handle)
{
	DeviceListEntry *pIterator, *pNext;

	IF_DBG(SLog(SLOG_LEVEL_INFO, "Removing Phidget from global attached list: handle=%i", handle));
	LOCK_MUTEX();

	if(g_phidgetManagerData.attachedDevices != NULL)
	{
		if(g_phidgetManagerData.attachedDevices->handle == handle) // is it the first entry?
		{
			// remove it
			pIterator = g_phidgetManagerData.attachedDevices;
			g_phidgetManagerData.attachedDevices = pIterator->pNext;

			// free its memory
			free(pIterator);
			g_phidgetManagerData.iListSize--;

			UNLOCK_MUTEX();
			return TRUE;
		}
		for(pIterator = g_phidgetManagerData.attachedDevices;
			pIterator != NULL;
			pIterator = pNext)
		{
			pNext = pIterator->pNext;
			if(pNext->handle == handle)
			{
				// remove next item
				pIterator->pNext = pNext->pNext;

				// free its memory
				free(pNext);
				g_phidgetManagerData.iListSize--;

				UNLOCK_MUTEX();
				return TRUE;
			}
		}
	}

	UNLOCK_MUTEX();
	IF_DBG(SLog(SLOG_LEVEL_WARNING, "Trying to remove Phidget from attached list, but it could not be found: handle=%i", handle));
	return FALSE;
}

// NULL if non existant
CPhidgetHandle phidgetManager_getItemAtIndex(int iIndex)
{
	DeviceListEntry *pIterator;
	int iCurrent;
	CPhidgetHandle handle;

	LOCK_MUTEX();

	if(iIndex >= 0 && iIndex < g_phidgetManagerData.iListSize)
	{
		for(pIterator = g_phidgetManagerData.attachedDevices, iCurrent = 0;
			pIterator != NULL;
			pIterator = pIterator->pNext, iCurrent++)
		{
			if(iCurrent == iIndex)
			{
				handle = pIterator->handle;
				UNLOCK_MUTEX();
				return handle;
			}
		}
	}

	UNLOCK_MUTEX();

	IF_DBG(SLog(SLOG_LEVEL_WARNING, "Trying to get attached phidget at index=%i, but no such exists", iIndex));
	return NULL;
}

// if list not initialised, it will be initialised
int phidgetManager_getNumberOfAttachedDevices()
{
	int iValue;

	LOCK_MUTEX();

	if(!g_phidgetManagerData.bListInitialised)
		phidgetManager_initaliseAttachedDevicesList();
	iValue = g_phidgetManagerData.iListSize;

	UNLOCK_MUTEX();
	return iValue;
}

// if list not initialised, it will stay uninitialised
BOOL phidgetManager_getAttachedDeviceName(int iIndex, const char** ppcName, int* piNameSize)
{
	CPhidgetHandle handle;

	handle = phidgetManager_getItemAtIndex(iIndex);
	if(handle == NULL || CPhidget_getDeviceName(handle, ppcName) != EPHIDGET_OK)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot get name of attached phidget, index=%i", iIndex));
		return FALSE;
	}

	*piNameSize = strlen(*ppcName); // without '\0'
	return TRUE;
}

// if list not initialised, it will stay uninitialised
BOOL phidgetManager_getAttachedDeviceSerialNumber(int iIndex, int* piSerialNumber)
{
	CPhidgetHandle handle;

	handle = phidgetManager_getItemAtIndex(iIndex);
	if(handle == NULL || CPhidget_getSerialNumber(handle, piSerialNumber) != EPHIDGET_OK)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot get serial number of attached phidget, index=%i", iIndex));
		return FALSE;
	}

	return TRUE;
}

// if list not initialised, it will stay uninitialised
BOOL phidgetManager_getAttachedDeviceLabel(int iIndex, const char** ppcLabel, int* piLabelSize)
{
	CPhidgetHandle handle;

	handle = phidgetManager_getItemAtIndex(iIndex);
	if(handle == NULL || CPhidget_getDeviceLabel(handle, ppcLabel) != EPHIDGET_OK)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot get label of attached phidget, index=%i", iIndex));
		return FALSE;
	}

	*piLabelSize = strlen(*ppcLabel); // without '\0'
	return TRUE;
}

// if list not initialised, it will stay uninitialised
BOOL phidgetManager_getAttachedDeviceClass(int iIndex, CPhidget_DeviceClass* pClass)
{
	CPhidgetHandle handle;

	handle = phidgetManager_getItemAtIndex(iIndex);
	if(handle == NULL || CPhidget_getDeviceClass(handle, pClass) != EPHIDGET_OK)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot get class of attached phidget, index=%i", iIndex));
		return FALSE;
	}

	return TRUE;
}

// if list not initialised, it will stay uninitialised
BOOL phidgetManager_getAttachedDeviceType(int iIndex, CPhidget_DeviceID* pDeviceID)
{
	CPhidgetHandle handle;

	handle = phidgetManager_getItemAtIndex(iIndex);
	if(handle == NULL || CPhidget_getDeviceID(handle, pDeviceID) != EPHIDGET_OK)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Cannot get type of attached phidget, index=%i", iIndex));
		return FALSE;
	}

	return TRUE;
}

// set and signal once
BOOL phidgetManager_SetSemaphoreIndex_Attach(int iSemaphore)
{
	LOCK_MUTEX();
	if(g_phidgetManagerData.iSemaphoreAttach != iSemaphore)
	{
		g_phidgetManagerData.iSemaphoreAttach = iSemaphore;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <PhidgetManager-Attach> is: %d", iSemaphore));
		if (iSemaphore > 0)
			interpreterProxy->signalSemaphoreWithIndex(iSemaphore);
	}
	UNLOCK_MUTEX();

	return TRUE;
}

// set and signal once
BOOL phidgetManager_SetSemaphoreIndex_Detach(int iSemaphore)
{
	LOCK_MUTEX();
	if(g_phidgetManagerData.iSemaphoreDetach != iSemaphore)
	{
		g_phidgetManagerData.iSemaphoreDetach = iSemaphore;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <PhidgetManager-Detach> is: %d", iSemaphore));
		if (iSemaphore > 0)
			interpreterProxy->signalSemaphoreWithIndex(iSemaphore);
	}
	UNLOCK_MUTEX();

	return TRUE;
}

// set and signal once
BOOL phidgetManager_SetSemaphoreIndex_Error(int iSemaphore)
{
	LOCK_MUTEX();
	if(g_phidgetManagerData.iSemaphoreError != iSemaphore)
	{
		g_phidgetManagerData.iSemaphoreError = iSemaphore;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <PhidgetManager-Error> is: %d", iSemaphore));
		if (iSemaphore > 0)
			interpreterProxy->signalSemaphoreWithIndex(iSemaphore);
	}
	UNLOCK_MUTEX();

	return TRUE;
}

// event handlers

int CALLBACK phidgetManager_attachHandler(CPhidgetHandle newPhidget, void* pUserPtr)
{
	int iSemaphore;

	LOCK_MUTEX();
	iSemaphore = g_phidgetManagerData.iSemaphoreAttach;
	UNLOCK_MUTEX();

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<PhidgetManager-Attach> semaphore = %i, handle = %i",
		iSemaphore, newPhidget));

	phidgetManager_addPhidgetToList(newPhidget);

	if (iSemaphore > 0)
		interpreterProxy->signalSemaphoreWithIndex(iSemaphore);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <PhidgetManager-Attach>"));
	return 0;
}

int CALLBACK phidgetManager_detachHandler(CPhidgetHandle oldPhidget, void* pUserPtr)
{
	int iSemaphore;

	LOCK_MUTEX();
	iSemaphore = g_phidgetManagerData.iSemaphoreDetach;
	UNLOCK_MUTEX();

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<PhidgetManager-Detach> semaphore = %i, handle = %i",
		iSemaphore, oldPhidget));

	phidgetManager_removePhidgetFromList(oldPhidget);

	if (iSemaphore > 0)
		interpreterProxy->signalSemaphoreWithIndex(iSemaphore);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <PhidgetManager-Detach>"));
	return 0;
}

int CALLBACK phidgetManager_errorHandler(CPhidgetManagerHandle manager, void* pUserPtr, int iError, const char* pcError)
{
	int iSemaphore;

	LOCK_MUTEX();
	iSemaphore = g_phidgetManagerData.iSemaphoreError;
	UNLOCK_MUTEX();

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<PhidgetManager-Error> semaphore = %i, code = %i, string = %s",
		iSemaphore, iError, pcError));

	if (iSemaphore > 0)
		interpreterProxy->signalSemaphoreWithIndex(iSemaphore);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <PhidgetManager-Error>"));
	return 0;
}
