/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgeRFID.c

Abstract:

	This file contains Phidget RFID specific methods. A Phidget RFID has a
	component rfid, led and output. These four modules - PhidgetCompRFID.c,
	PhidgetCompLED.c, PhidgetCompDigitalOutput.c and PhidgetRFID.c - 
	inmplement all methods supplied by the phidget21 API for an RFID.

--*/

#include "PhidgetRFID.h"

//////////////////////////////////////////////////////////////////////////
// forward declarations: callback functions

int CALLBACK phidgetRFID_outputChangeHandler(CPhidgetRFIDHandle handle, void *pUserPtr, int iIndex, int iState);
int CALLBACK phidgetRFID_tagFoundHandler(CPhidgetRFIDHandle handle, void *pUserPtr, unsigned char *pcTag);
int CALLBACK phidgetRFID_tagLostHandler(CPhidgetRFIDHandle handle, void *pUserPtr, unsigned char *pcTag);

//////////////////////////////////////////////////////////////////////////
// PhidgetRFID methods

/*++

Routine Description:

	This method creates an PhidgetRFID struct and allocates memory.
	To identify the phidgets type the deviceClass is set here.
	Additionally do common creational settings.

Arguments:

	None.

Return Value:

	Squeak VM identifier for the PhigdetId Object.

--*/
PhidgetId phidgetRFID_Create()
{
	// allocate memory for a PhidgetRFID structure
	PhidgetRFID* pPhidget = calloc(1, sizeof(PhidgetRFID));

	// initialise
	pPhidget->deviceClass = PHIDCLASS_RFID;
	phidgetRFID_ResetSemaphores(pPhidget);

	// get id and add to global list. return the id
	return phidget_Create((Phidget*)pPhidget);
}

//
//reset semaphores
//
void phidgetRFID_ResetSemaphores(PhidgetRFID *pPhidget)
{
	if(pPhidget != NULL)
	{
		phidget_ResetSemaphores((Phidget*)pPhidget);
		
		pPhidget->iSemaphoreOutputChange = 0;
		pPhidget->iSemaphoreTagFound = 0;
		pPhidget->iSemaphoreTagLost = 0;
	}
	else
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetRFID_ResetSemaphores for pPhidget."));
}


/*++

Routine Description:

	This method is called while opening a phidget. The actuall specific
	phidget object will be created and the specific callbacks will be
	registered.

Arguments:

	pPhidget - pointer of the Phidget to open the hardware connection.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
int phidgetRFID_Open(PhidgetRFID *pPhidget)
{
	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetRFID_Open for pPhidget."));
		return FALSE;
	}

	//create the PhidgetRFID object
	pPhidget->handle=0;
	CPhidgetRFID_create(&(pPhidget->handle));

	//Registers the callbacks that will run if an ouput changes or if we found or lost a tag.
	//Requires the handle for the Phidget, the function that will be called, and an arbitrary pointer that will be supplied to the callback function (may be NULL).
	CPhidgetRFID_set_OnOutputChange_Handler (pPhidget->handle, phidgetRFID_outputChangeHandler, (void *)pPhidget->id);
	CPhidgetRFID_set_OnTag_Handler (pPhidget->handle, phidgetRFID_tagFoundHandler, (void *)pPhidget->id);
	CPhidgetRFID_set_OnTagLost_Handler (pPhidget->handle, phidgetRFID_tagLostHandler, (void *)pPhidget->id);

	// do phidget common open operations
	phidget_Open_Base((Phidget *)pPhidget);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// callback functions

/*++

Routine Description:

	This callback is called when an output of a Phidget RFID with
	the id pUserPtr has changed.
	It signals the semaphore iSemaphoreOutputChange.

Arguments:

	rfidHandle - handle of the RFID to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the output that generated the event

	iState - the output state (on or off)

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetRFID_outputChangeHandler(CPhidgetRFIDHandle rfidHandle, void *pUserPtr, int iIndex, int iState)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetRFID* pPhidget = (PhidgetRFID*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<RFID-OutputChange> id = %i, semaphore = %i, index = %i, value = %i",
		pPhidget->id, pPhidget->iSemaphoreOutputChange, iIndex, iState));
	
	if (pPhidget->iSemaphoreOutputChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreOutputChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <RFID-OutputChange>, id = %i", pPhidget->id));
	return 0;
}

/*++

Routine Description:

	Helper function that concats an character array of size
	4 or smaller to an integer value.
	Used for printing the found RFID tag to the log in
	phidgetRFID_tagFoundHandler and phidgetRFID_tagLostHandler.

Arguments:

	pcArray - Byte array to be interpreted as an integer and
		to be concatenated.

	iSize - Size of the latter array with 0 <= size <= 4.

Return Value:

	Integer value representing the array.

--*/
int concatArrayToNumber(unsigned char* pcArray, int iSize)
{
	int i;
	int result = 0;

	for(i = 0; i < iSize; i++)
	{
		result = (result << 8) | (unsigned int)pcArray[i];
	}
	return result;
}

/*++

Routine Description:

	This callback is called when a Phidget RFID with the id pUserPtr gets
	a signal from one tag.
	It signals the semaphore iSemaphoreTagFound.

Arguments:

	rfidHandle - handle of the RFID to signal the semaphore.

	pUserPtr - Callback function pointer.

	pcTag - string representing the tag ID

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetRFID_tagFoundHandler(CPhidgetRFIDHandle rfidHandle, void *pUserPtr, unsigned char *pcTag)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetRFID* pPhidget = (PhidgetRFID*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<RFID-TagFound> id = %i, semaphore = %i, value = %08x%02x",
		pPhidget->id, pPhidget->iSemaphoreTagFound, concatArrayToNumber(pcTag, 4), pcTag[4]));
	
	if (pPhidget->iSemaphoreTagFound > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreTagFound);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <RFID-TagFound>, id = %i", pPhidget->id));
	return 0;
}

/*++

Routine Description:

	This callback is called when a Phidget RFID with the id pUserPtr lost
	signal to a tag.
	It signals the semaphore iSemaphoreTagLost.

Arguments:

	rfidHandle - handle of the RFID to signal the semaphore.

	pUserPtr - Callback function pointer.

	pcTag - string representing the tag ID

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetRFID_tagLostHandler(CPhidgetRFIDHandle handle, void *pUserPtr, unsigned char *pcTag)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetRFID* pPhidget = (PhidgetRFID*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<RFID-TagLost> id = %i, semaphore = %i, value = %08x%02x",
		pPhidget->id, pPhidget->iSemaphoreTagLost, concatArrayToNumber(pcTag, 4), pcTag[4]));
	
	if (pPhidget->iSemaphoreTagLost > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreTagLost);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <RFID-TagLost>, id = %i", pPhidget->id));
	return 0;
}
